@@ -23,32 +23,38 @@
 #define _GNU_SOURCE
 
 #include <string.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif /* HAVE_ALLOCA_H */
 #include <unistd.h>
+#include <stdlib.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <time.h>
 #include "yp.h"
 #include "yp_db.h"
 #include "access.h"
 #include "ypserv_conf.h"
 #include "log_msg.h"
+#include "compat.h"
 
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
 extern volatile int children; /* ypserv.c  */
-extern int forked; /* ypserv.c  */
+#endif
 
 bool_t
-ypproc_null_2_svc (void *argp __attribute__ ((unused)),
-		   void *result __attribute__ ((unused)),
+ypproc_null_2_svc (void *argp UNUSED, void *result UNUSED,
 		   struct svc_req *rqstp)
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_null() [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
@@ -67,7 +73,7 @@ ypproc_domain_2_svc (domainname *argp, bool_t *result,
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_domain(\"%s\") [From: %s:%d]",
 	       *argp, inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
@@ -102,7 +108,7 @@ ypproc_domain_nonack_2_svc (domainname *argp, bool_t *result,
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_domain_nonack(\"%s\") [From: %s:%d]",
 	       *argp, inet_ntoa (rqhost->sin_addr),
 	       ntohs (rqhost->sin_port));
@@ -116,6 +122,7 @@ ypproc_domain_nonack_2_svc (domainname *argp, bool_t *result,
       /* Bail out and don't return any RPC value */
       return FALSE;
     case -2: /* Should not happen */
+      log_msg ("Map name not valid, this cannot happen???");
       return FALSE;
     case -1:
       if (debug_flag)
@@ -141,7 +148,7 @@ ypproc_match_2_svc (ypreq_key *argp, ypresp_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_match(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -229,7 +236,7 @@ ypproc_first_2_svc (ypreq_nokey *argp, ypresp_key_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_first(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
 
@@ -324,7 +331,7 @@ ypproc_next_2_svc (ypreq_key *argp, ypresp_key_val *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_next(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -421,7 +428,7 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
 		  struct svc_req *rqstp)
 {
   DB_FILE dbp;
-  struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+  const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
   int valid;
 
   if (debug_flag)
@@ -506,7 +513,7 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
           if ((size_t)val.dsize != strlen (argp->map_parms.peer) ||
               strncmp (val.dptr, argp->map_parms.peer, val.dsize) != 0)
             {
-	      char buf[val.dsize + 1];
+	      char *buf = alloca (val.dsize + 1);
 
 	      strncpy (buf, val.dptr, val.dsize);
 	      buf[val.dsize] = '\0';
@@ -572,7 +579,9 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
     }
 #endif
 
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
   ++children;
+#endif
   switch (fork ())
     {
     case 0:
@@ -587,9 +596,9 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
         dup (i);
 
         sprintf (ypxfr_command, "%s/ypxfr", YPBINDIR);
-        sprintf (t, "%u", argp->transid);
-        sprintf (g, "%u", argp->prog);
-        sprintf (p, "%u", argp->port);
+        snprintf (t, sizeof (t), "%u", argp->transid);
+        snprintf (g, sizeof (g), "%u", argp->prog);
+        snprintf (p, sizeof (p), "%u", argp->port);
         if (debug_flag)
           execl (ypxfr_command, "ypxfr", "--debug", "-d",
                  argp->map_parms.domain, "-h", argp->map_parms.peer,
@@ -604,7 +613,9 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
         exit (0);
       }
     case -1:
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
       --children;
+#endif
       log_msg ("Cannot fork: %s", strerror (errno));
       result->xfrstat = YPXFR_XFRERR;
       break;
@@ -616,13 +627,12 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
   return TRUE;
 }
 
-bool_t ypproc_clear_2_svc (void *argp __attribute__ ((unused)),
-			   void *result __attribute__ ((unused)),
+bool_t ypproc_clear_2_svc (void *argp UNUSED, void *result UNUSED,
 			   struct svc_req *rqstp)
 {
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_clear_2_svc() [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
     }
@@ -724,7 +734,7 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost;
+      const struct sockaddr_in *rqhost;
 
       rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_all_2_svc(): [From: %s:%d]",
@@ -739,6 +749,10 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
   xdr_ypall_cb.u.close = NULL;
   xdr_ypall_cb.data = NULL;
 
+  /* Set this to TRUE so that the client will be forced to read
+     at least one record from us. This could be the error code.  */
+  result->more = TRUE;
+
   valid = is_valid (rqstp, argp->map, argp->domain);
   if (valid < 1)
     {
@@ -762,6 +776,7 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
       return TRUE;
     }
 
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
   if (children >= MAX_CHILDREN)
     {
       int wait = 0;
@@ -779,28 +794,27 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
   if (children < MAX_CHILDREN)
     {
       ++children;
+#endif
       switch (fork ())
         {
-        case 0:
-          ++forked;
+        case 0: /* child */
 #ifdef DEBUG
           log_msg ("ypserv has forked for ypproc_all(): pid=%i", getpid ());
-          if (!forked)
-            abort ();
 #endif
-          /* Close all databases ! */
-          ypdb_close_all ();
           break;
-        case -1:
+        case -1:  /* parent, error */
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
           --children;
+#endif
           log_msg ("WARNING(ypproc_all_2_svc): cannot fork: %s",
 		   strerror (errno));
           result->ypresp_all_u.val.stat = YP_YPERR;
-          return TRUE;
-        default:
+	  return TRUE;
+        default: /* parent, default */
           return FALSE;
           break;
         }
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
     }
   else
     {
@@ -808,19 +822,18 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
       result->ypresp_all_u.val.stat = YP_YPERR;
       return TRUE;
     }
+#endif
 
   /* We are now in the child part. Don't let the child ypserv share
      DB handles with the parent process.  */
   ypdb_close_all();
 
-  result->more = TRUE;
-
   if ((data = calloc (1, sizeof (struct ypall_data))) == NULL)
     {
       log_msg ("ERROR: could not allocate enough memory! [%s|%d]",
 	       __FILE__, __LINE__);
       result->ypresp_all_u.val.stat = YP_YPERR;
-      return TRUE;
+      goto out;
     }
 
   data->dbm = ypdb_open (argp->domain, argp->map);
@@ -859,9 +872,9 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
 	    log_msg ("\t -> First value returned.");
 
 	  if (result->ypresp_all_u.val.stat == YP_TRUE)
-	    return TRUE; /* We return to commit the data.
-			    This also means, we don't give
-			    data free here */
+	    goto out; /* We return to commit the data.
+			 This also means, we don't give
+			 data free here */
 	}
       else
 	result->ypresp_all_u.val.stat = YP_NOMORE;
@@ -874,7 +887,12 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
   if (debug_flag)
     log_msg ("\t -> Exit from ypproc_all without sending data.");
 
-  return TRUE;
+ out:
+  if (!svc_sendreply (rqstp->rq_xprt, (xdrproc_t) xdr_ypresp_all,
+		      (caddr_t) result))
+    svcerr_systemerr (rqstp->rq_xprt);
+  /* Note: no need to free args; we're exiting.  */
+  exit(0);
 }
 
 bool_t
@@ -886,7 +904,7 @@ ypproc_master_2_svc (ypreq_nokey *argp, ypresp_master *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
       log_msg ("ypproc_master_2_svc(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
 
@@ -993,7 +1011,7 @@ ypproc_order_2_svc (ypreq_nokey *argp, ypresp_order *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost;
+      const struct sockaddr_in *rqhost;
 
       rqhost = svc_getcaller (rqstp->rq_xprt);
 
@@ -1117,7 +1135,7 @@ ypproc_maplist_2_svc (domainname *argp, ypresp_maplist *result,
 
   if (debug_flag)
     {
-      struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+      const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
       log_msg ("ypproc_maplist_2_svc(): [From: %s:%d]",
 	       inet_ntoa (rqhost->sin_addr), ntohs (rqhost->sin_port));
@@ -1200,7 +1218,7 @@ ypproc_maplist_2_svc (domainname *argp, ypresp_maplist *result,
 }
 
 int
-ypprog_2_freeresult (SVCXPRT *transp __attribute__ ((unused)),
+ypprog_2_freeresult (SVCXPRT *transp UNUSED,
 		     xdrproc_t xdr_result, caddr_t result)
 {
   xdr_free (xdr_result, result);
