@@ -29,8 +29,9 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <signal.h>
+#if defined(HAVE_GETOPT_H)
 #include <getopt.h>
-#include <poll.h>
+#endif
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -42,6 +43,7 @@
 #include "access.h"
 #include "log_msg.h"
 #include "ypserv_conf.h"
+#include "compat.h"
 
 #ifdef HAVE_PATHS_H
 #include <paths.h>
@@ -55,8 +57,9 @@
 #define YPOLDVERS 1
 #endif
 
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
 volatile int children = 0;
-int forked = 0;
+#endif
 
 static char *path_ypdb = YPMAPDIR;
 
@@ -204,47 +207,6 @@ ypprog_2 (struct svc_req *rqstp, register SVCXPRT * transp)
   return;
 }
 
-static void
-ypserv_svc_run (void)
-{
-  int i;
-
-  for (;;)
-    {
-      struct pollfd *my_pollfd;
-
-      if (svc_max_pollfd == 0 && svc_pollfd == NULL)
-        return;
-
-      my_pollfd = malloc (sizeof (struct pollfd) * svc_max_pollfd);
-      for (i = 0; i < svc_max_pollfd; ++i)
-        {
-          my_pollfd[i].fd = svc_pollfd[i].fd;
-          my_pollfd[i].events = svc_pollfd[i].events;
-          my_pollfd[i].revents = 0;
-        }
-
-      switch (i = poll (my_pollfd, svc_max_pollfd, -1))
-        {
-        case -1:
-          free (my_pollfd);
-          if (errno == EINTR)
-            continue;
-          syslog (LOG_ERR, "svc_run: - poll failed: %m");
-          return;
-        case 0:
-          free (my_pollfd);
-          continue;
-        default:
-          svc_getreq_poll (my_pollfd, i);
-          free (my_pollfd);
-          if (forked)
-            _exit (0);
-        }
-    }
-}
-
-
 /* Create a pidfile on startup */
 static void
 create_pidfile (void)
@@ -325,9 +287,29 @@ create_pidfile (void)
   return;
 }
 
+extern FILE *debug_output;
+/* SIGUSR1: enable/disable debug output.  */
+static void
+sig_usr1 (int sig UNUSED)
+{
+  if (debug_flag)
+    {
+      debug_flag = 0;
+      if (debug_output != stderr)
+	fclose (debug_output);
+      debug_output = stderr;
+    }
+  else
+    {
+      debug_output = fopen ("/var/yp/ypserv.log", "a");
+      if (debug_output != NULL)
+	debug_flag = 1;
+    }
+}
+
 /* Clean up if we quit the program. */
 static void
-sig_quit (int sig __attribute__ ((unused)))
+sig_quit (int sig UNUSED)
 {
   pmap_unset (YPPROG, YPVERS);
   pmap_unset (YPPROG, YPOLDVERS);
@@ -338,7 +320,7 @@ sig_quit (int sig __attribute__ ((unused)))
 
 /* Reload securenets and config file */
 static void
-sig_hup (int sig __attribute__ ((unused)))
+sig_hup (int sig UNUSED)
 {
   int old_cached_filehandles = cached_filehandles;
 
@@ -366,13 +348,17 @@ sig_child (int sig)
     {
       if (debug_flag)
         log_msg ("pid=%d", pid);
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
       --children;
+#endif
     }
 
+#if defined (MAX_CHILDREN) && MAX_CHILDREN > 0
   if (children < 0)
     log_msg ("children is lower 0 (%i)!", children);
   else if (debug_flag)
     log_msg ("children = %i", children);
+#endif
 }
 
 
@@ -525,6 +511,14 @@ main (int argc, char **argv)
   sigemptyset (&sa.sa_mask);
   sigaction (SIGHUP, &sa, NULL);
 
+  /*
+   * If we get a SIGUSR1, enable/disable debuging.
+   */
+  sigaction (SIGUSR1, NULL, &sa);
+  sa.sa_handler = sig_usr1;
+  sigemptyset (&sa.sa_mask);
+  sigaction (SIGUSR1, &sa, NULL);
+
   /*
    * On SIGCHLD wait for the child process, so it can give free all
    * resources.
@@ -627,7 +621,7 @@ main (int argc, char **argv)
       exit (1);
     }
 
-  ypserv_svc_run ();
+  svc_run ();
   log_msg ("svc_run returned");
   unlink (_YPSERV_PIDFILE);
   exit (1);
